<?php

/**
 * Action class for rtgrid_model
 */
class rtgrid_action extends rtgrid_top_action
{

    public function get_template($row = array())
    {
        if (empty($this->template)) {

            return '<a href="#" onclick="doAction(\'' . $this->name . '\', \'' . $this->message . '\', $(this), false);return false;">' . $this->text . '</a>';
        } else {

            return $this->_prepare_template($row);
        }
    }

}